/*
 * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
 * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/*
 * $Id: URLClient.java 72229 2013-06-25 18:20:40Z dougd $
 */

package com.sun.ts.tests.jsf.api.javax_faces.convert.byteconverter;

import com.sun.javatest.Status;
import com.sun.ts.tests.jsf.common.client.AbstractUrlClient;

import java.io.PrintWriter;

public final class URLClient extends AbstractUrlClient {

	private static final String TYPE = "byte";
	private static final String CONTEXT_ROOT = "/jsf_convert_byte_web";

	public static void main(String[] args) {
		URLClient theTests = new URLClient();
		Status s = theTests.run(args, new PrintWriter(System.out),
				new PrintWriter(System.err));
		s.exit();
	}

	public Status run(String args[], PrintWriter out, PrintWriter err) {
		setContextRoot(CONTEXT_ROOT);
		setServletName(DEFAULT_SERVLET_NAME);
		return super.run(args, out, err);
	}

	/*
	 * @class.setup_props: webServerHost; webServerPort; ts_home;
	 */

	/* Test Declarations */

	/**
	 * @testName: byteConverterGetAsObjectTest
	 * 
	 * @assertion_ids: JSF:JAVADOC:1560; JSF:JAVADOC:1561; JSF:JAVADOC:1574
	 * 
	 * @test_Strategy: Ensure the proper result is returned when passing in a
	 *                 valid value, null, or a zero-length String.
	 */
	public void byteConverterGetAsObjectTest() throws Fault {
		TEST_PROPS
				.setProperty(APITEST, "converterGetAsObjectTest&type=" + TYPE);
		invoke();
	}

	/**
	 * @testName: byteConverterGetAsObjectNPETest
	 * 
	 * @assertion_ids: JSF:JAVADOC:1560; JSF:JAVADOC:1563; JSF:JAVADOC:1576
	 * 
	 * @test_Strategy: Ensure an NPE is thrown if either the context or
	 *                 component parameters are null.
	 */
	public void byteConverterGetAsObjectNPETest() throws Fault {
		TEST_PROPS.setProperty(APITEST, "converterGetAsObjectNPETest&type="
				+ TYPE);
		invoke();
	}

	/**
	 * @testName: byteConverterGetAsStringTest
	 * 
	 * @assertion_ids: JSF:JAVADOC:1560; JSF:JAVADOC:1564; JSF:JAVADOC:1577
	 * 
	 * @test_Strategy: Ensure the proper result is returned when passing in a
	 *                 valid value, null, or a zero-length String.
	 */
	public void byteConverterGetAsStringTest() throws Fault {
		TEST_PROPS
				.setProperty(APITEST, "converterGetAsStringTest&type=" + TYPE);
		invoke();
	}

	/**
	 * @testName: byteConverterGetAsStringNPETest
	 * 
	 * @assertion_ids: JSF:JAVADOC:1560; JSF:JAVADOC:1566; JSF:JAVADOC:1579
	 * 
	 * @test_Strategy: Ensure an NPE is thrown if either the context or
	 *                 component parameters are null.
	 */
	public void byteConverterGetAsStringNPETest() throws Fault {
		TEST_PROPS.setProperty(APITEST, "converterGetAsStringNPETest&type="
				+ TYPE);
		invoke();
	}

} // end of URLClient
